@@ -26,7 +26,9 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   portalConsts
 );
 
-our $VERSION = '2.19.0';
+use Lemonldap::NG::Common::Util qw/display2F filterKey2F/;
+
+our $VERSION = '2.20.0';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 with qw(
@@ -85,46 +87,7 @@ sub init {
             }
           )
         {
-            my $prefix = lc($_);
-            $prefix =~ s/2f$//i;
-
-            # Activation parameter
-            my $ap = $prefix . ( $i ? '2fSelfRegistration' : '2fActivation' );
-            $self->logger->debug("Checking $ap");
-
-            # Unless $rule, skip loading
-            if ( $self->conf->{$ap} ) {
-                $self->logger->debug("Trying to load $_ 2F");
-                my $m =
-                  $self->p->loadPlugin( $i ? "::2F::Register::$_" : "::2F::$_" )
-                  or return 0;
-
-                $self->_registerRoutes($m) unless ($i);
-
-                # Rule and prefix may be modified by 2F module, reread them
-                my $rule = $self->conf->{$ap};
-                $prefix = $m->prefix;
-
-                # Compile rule
-                $rule = $self->p->HANDLER->substitute($rule);
-                unless ( $rule = $self->p->HANDLER->buildSub($rule) ) {
-                    $self->error( 'External 2F rule error: '
-                          . $self->p->HANDLER->tsv->{jail}->error );
-                    return 0;
-                }
-
-                # Store module
-                push @{ $self->{ $i ? 'sfRModules' : 'sfModules' } },
-                  {
-                    p => $prefix,
-                    m => $m,
-                    r => $rule,
-                    t => ( $m->can('type') ? $m->type : $prefix ),
-                  };
-            }
-            else {
-                $self->logger->debug(' -> not enabled');
-            }
+            return 0 unless $self->_load_2f_module_by_prefix( $_, $i );
         }
     }
 
@@ -138,7 +101,7 @@ sub init {
 
         $self->logger->debug(
             "Loading extra 2F module $extraKey of type $moduleType");
-        my $m = $self->loadPlugin(
+        my $m = $self->loadModule(
             "::2F::$moduleType",
             $over,
             prefix => $extraKey,
@@ -187,10 +150,11 @@ sub init {
                 "Loading register module for $extraKey of type $moduleType");
             my $register_module_name = $self->findRegisterModuleFor($m);
             if ($register_module_name) {
-                my $rm = $self->loadPlugin(
+                my $rm = $self->loadModule(
                     "::2F::Register::Generic",
                     $over,
                     prefix             => $extraKey,
+                    type               => $extraKey,
                     logo               => $m->logo,
                     label              => $m->label,
                     userCanRemove      => 1,
@@ -237,29 +201,95 @@ sub init {
         return 0;
     }
 
-    # Enable REST request only if more than 1 2F module is enabled
-    if ( @{ $self->{sfModules} } > 1 ) {
-        $self->addAuthRoute( '2fchoice' => '_choice',   ['POST'] );
-        $self->addAuthRoute( '2fchoice' => '_redirect', ['GET'] );
-        $self->addUnauthRoute( '2fchoice' => '_choice',   ['POST'] );
-        $self->addUnauthRoute( '2fchoice' => '_redirect', ['GET'] );
-    }
+    $self->addAuthRoute( '2fchoice' => '_choice',   ['POST'] );
+    $self->addAuthRoute( '2fchoice' => '_redirect', ['GET'] );
+    $self->addUnauthRoute( '2fchoice' => '_choice',   ['POST'] );
+    $self->addUnauthRoute( '2fchoice' => '_redirect', ['GET'] );
 
-    # Enable 2F registration URL only if at least 1 registration module
-    # is enabled
-    if ( @{ $self->{sfRModules} } ) {
+    # Registration base
+    $self->addAuthRoute( '2fregisters' => '_displayRegister', ['GET'] );
+    $self->addAuthRoute( '2fregisters' => 'register',         ['POST'] );
+    $self->addUnauthRoute(
+        '2fregisters' => 'restoreSession',
+        [ 'GET', 'POST' ]
+    ) if ( $self->conf->{sfRequired} );
 
-        # Registration base
-        $self->addAuthRoute( '2fregisters' => '_displayRegister', ['GET'] );
-        $self->addAuthRoute( '2fregisters' => 'register',         ['POST'] );
-        $self->addUnauthRoute(
-            '2fregisters' => 'restoreSession',
-            [ 'GET', 'POST' ]
-        ) if ( $self->conf->{sfRequired} );
+    $self->addEntryPoint(
+        isa     => 'Lemonldap::NG::Portal::Main::SecondFactor',
+        service => "secondFactor",
+        method  => "_enable_2f_module",
+    );
+
+    $self->addEntryPoint(
+        isa     => 'Lemonldap::NG::Portal::2F::Register::Base',
+        service => "secondFactor",
+        method  => "_enable_2f_register_module",
+    );
+
+    return 1;
+}
+
+sub _load_2f_module_by_prefix {
+    my ( $self, $name, $isRegister ) = @_;
+    {
+        my $prefix = lc($name);
+        $prefix =~ s/2f$//i;
+
+        # Activation parameter
+        my $ap =
+          $prefix . ( $isRegister ? '2fSelfRegistration' : '2fActivation' );
+        $self->logger->debug("Checking $ap");
+
+        # Unless $rule, skip loading
+        if ( $self->conf->{$ap} ) {
+            $self->logger->debug("Trying to load $name 2F");
+            my $m =
+              $self->p->loadModule(
+                $isRegister ? "::2F::Register::$name" : "::2F::$name" )
+              or return 0;
+
+            if ($isRegister) {
+                $self->_enable_2f_register_module($m) or return 0;
+            }
+            else {
+                $self->_enable_2f_module($m) or return 0;
+            }
+        }
+        else {
+            $self->logger->debug(' -> not enabled');
+        }
     }
     return 1;
 }
 
+sub _enable_2f_register_module {
+    my ( $self, $m ) = @_;
+    return $self->_enable_module( $m, $self->sfRModules );
+}
+
+sub _enable_2f_module {
+    my ( $self, $m ) = @_;
+
+    $self->_registerRoutes($m);
+    return $self->_enable_module( $m, $self->sfModules );
+}
+
+sub _enable_module {
+    my ( $self, $m, $array ) = @_;
+
+    my $rule = $self->p->buildRule( $m->rule );
+    return 0 unless $rule;
+
+    push @$array,
+      {
+        p => $m->prefix,
+        m => $m,
+        r => $rule,
+        t => ( $m->can('type') ? $m->type : $m->prefix ),
+      };
+    return 1;
+}
+
 # RUNNING METHODS
 
 # public PE_CODE run($req)
@@ -433,7 +463,7 @@ sub run {
             message => (
                     'Second factor '
                   . $am[0]->prefix
-                  . '2F selected for '
+                  . '2f selected for '
                   . $req->sessionInfo->{ $self->conf->{whatToTrace} }
             ),
             code => "2FA_SELECTED",
@@ -609,13 +639,19 @@ sub _displayRegister {
           unless $m;
         return $self->p->sendError( $req, 'Registration not authorized', 403 )
           unless $m->{r}->( $req, $req->userData );
+        my $mod = $m->{m};
+
+        if ( $mod->can('initDisplay') ) {
+            $mod->initDisplay($req);
+        }
         return $self->p->sendHtml(
             $req,
-            $m->{m}->template,
+            $mod->template,
             params => {
+                CUSTOM_SCRIPT    => $req->data->{customScript},
                 PREFIX           => $prefix,
                 "PREFIX_$prefix" => 1,
-                MSG   => $self->canUpdateSfa($req) || $m->{m}->welcome,
+                MSG              => $self->canUpdateSfa($req) || $mod->welcome,
                 ALERT => ( $self->canUpdateSfa($req) ? 'warning' : 'positive' ),
             }
         );
@@ -665,8 +701,16 @@ sub _displayRegister {
         my $prefix = $reg_mod_info->{p};
         my $module = $reg_mod_info->{m};
         foreach (@$_2fDevices) {
+
             if ( $_->{type} eq $type ) {
 
+                # Set k_v template variables for non_private values
+                # and also not for epoch/name
+                my $extrakeys = filterKey2F( $_, "epoch", "name" );
+                while ( my ( $k, $v ) = each %$extrakeys ) {
+                    $_->{"${k}_${v}"} = 1;
+                }
+
                 # Populate additional info for template engine
                 $_->{prefix}     = $prefix;
                 $_->{label}      = $module->label;
@@ -698,6 +742,8 @@ sub _displayRegister {
                 $self->p->buildUrl( $req->portal, '2fregisters' ), ''
             ),
             %tplParams,
+            CSRF_TOKEN => $self->ott->createToken()
+
         }
     );
 }
@@ -906,49 +952,98 @@ sub _verify {
     if ( $tokres != PE_OK ) {
         $self->auditLog(
             $req,
-            message      => "Token validation failed during 2FA verification",
-            code         => "2FA_VERIFICATION_FAILED",
-            user         => $req->sessionInfo->{ $self->conf->{whatToTrace} },
+            message => (
+                "Token validation failed during ${prefix}2f verification ("
+                  . portalConsts->{$tokres} . ")"
+            ),
+            code         => "2FA_FAILED",
             reason       => "Token validation failed",
+            type         => $prefix,
             portal_error => portalConsts->{$tokres},
         );
         return $self->p->do( $req, [ sub { $tokres } ] );
     }
 
+    my $user = $session->{ $self->conf->{whatToTrace} };
+
     # Evaluate hook
-    my $h = $self->p->processHook( $req, 'sfBeforeVerify', $module, $session );
-    if ( $h != PE_OK ) {
-        $req->noLoginDisplay(1);
-        return $self->p->do( $req, [ sub { $h } ] );
+    {
+        my $h =
+          $self->p->processHook( $req, 'sfBeforeVerify', $module, $session );
+        if ( $h != PE_OK ) {
+            $req->noLoginDisplay(1);
+            return $self->p->do( $req, [ sub { $h } ] );
+        }
     }
 
     # Launch second factor verification
-    my $res = $module->verify( $req, $session );
+    my $verify_result = {
+        result              => $module->verify( $req, $session ),
+        authenticationLevel => $module->authnLevel,
+        retries             => ( $session->{_2fRetries} || 0 ),
+        (
+            $req->data->{_2fDevice} ? ( device => $req->data->{_2fDevice} ) : ()
+        ),
+        (
+            $req->data->{_2fLogInfo}
+            ? ( logInfo => $req->data->{_2fLogInfo} )
+            : ()
+        ),
+    };
+
+    # Evaluate hook
+    {
+        my $h = $self->p->processHook( $req, 'sfAfterVerify', $module, $session,
+            $verify_result );
+        if ( $h != PE_OK ) {
+            $req->noLoginDisplay(1);
+            return $self->p->do( $req, [ sub { $h } ] );
+        }
+    }
+
+    # May be modified by sfAfterVerify
+    my $res        = $verify_result->{result};
+    my $authnLevel = $verify_result->{authenticationLevel};
+    my $retries    = $verify_result->{retries};
 
     $self->restore2faSessionInfo( $req, $session );
 
     # Case error
-    if ( $res and !$req->data->{_2fRetries} ) {
+    if ( $res and $retries == 0 ) {
         $req->noLoginDisplay(1);
         $req->authResult(PE_BADCREDENTIALS);
 
+        $self->auditLog(
+            $req,
+            message => (
+                "Second factor ${prefix}2f failed for $user ("
+                  . portalConsts->{$res} . ")"
+            ),
+            code         => "2FA_FAILED",
+            user         => $user,
+            type         => $prefix,
+            portal_error => portalConsts->{$res},
+        );
+
         # Set the failed 2f type in case you want to store it in history
         # through sessionDataToRemember
-        $req->sessionInfo->{'_2f'} = $module->prefix;
+        $req->sessionInfo->{'_2f'} = $prefix;
         return $self->p->do( $req, [ 'storeHistory', sub { $res } ] );
     }
     elsif ($res) {
         $req->data->{_2fRetries} = (
-            ( $req->data->{_2fRetries} > 0 )
-            ? $req->data->{_2fRetries} - 1
+            ( $retries > 0 )
+            ? $retries - 1
             : 0
         );
 
         # Evaluate hook
-        my $h = $self->p->processHook( $req, 'sfBeforeRetry', $module );
-        if ( $h != PE_OK ) {
-            $req->noLoginDisplay(1);
-            return $self->p->do( $req, [ sub { $h } ] );
+        {
+            my $h = $self->p->processHook( $req, 'sfBeforeRetry', $module );
+            if ( $h != PE_OK ) {
+                $req->noLoginDisplay(1);
+                return $self->p->do( $req, [ sub { $h } ] );
+            }
         }
 
         $self->logger->debug(
@@ -964,12 +1059,26 @@ sub _verify {
 
     # Else restore session
     $req->mustRedirect(1);
-    $self->userLogger->notice( $module->prefix
-          . '2f verification succeeded for '
-          . $req->sessionInfo->{ $self->conf->{whatToTrace} } );
+    my $device  = $verify_result->{device};
+    my $logInfo = $verify_result->{logInfo} || {};
+    my $logInfoStr =
+      join( ', ', map { "$_: " . $logInfo->{$_} } keys %$logInfo );
 
-    my $level = $module->authnLevel;
-    return $self->_continueLogin( $req, $module->prefix, $module->authnLevel );
+    $self->auditLog(
+        $req,
+        message => (
+                "Second factor ${prefix}2f verification succeeded for abarnes"
+              . ( $device     ? ( " with device " . display2F($device) ) : "" )
+              . ( $logInfoStr ? " ($logInfoStr)"                         : "" )
+        ),
+        code => "2FA_SUCCESS",
+        user => $user,
+        type => $prefix,
+        ( $device ? ( device => display2F($device) ) : () ),
+        %$logInfo,
+    );
+
+    return $self->_continueLogin( $req, $prefix, $authnLevel, );
 }
 
 sub searchForAuthorized2Fmodules {
@@ -977,10 +1086,16 @@ sub searchForAuthorized2Fmodules {
     $session ||= $req->sessionInfo;
     my @am;
     foreach ( @{ $self->sfModules } ) {
-        $self->logger->debug(
-            'Looking if ' . $_->{m}->prefix . '2f is available' );
-        if ( $_->{r}->( $req, $session ) ) {
-            $self->logger->debug(' -> OK');
+        my $authModuleName = $_->{m}->prefix;
+        $self->logger->debug('Looking for ' . $authModuleName . ' 2f');
+
+        # Adding targetAuthnLevel from req to session;
+        my $modifiedSession = {%$session};
+        $modifiedSession->{targetAuthnLevel} = $req->{pdata}->{targetAuthnLevel}
+          if $req->{pdata}->{targetAuthnLevel};
+
+        if ( $_->{r}->( $req, $modifiedSession ) ) {
+            $self->logger->debug(' -> ' . $authModuleName . ' 2f is available');
             push @am, $_->{m};
         }
     }
@@ -1144,7 +1259,7 @@ sub _registerRoutes {
 
 sub get2fTplParams {
     my ( $self, $req, $module ) = @_;
-    my %param;
+    my %param       = $self->p->getErrorTplParams($req);
     my $checkLogins = $req->param('checkLogins');
     $self->logger->debug( $module->prefix . '2f: checkLogins set' )
       if $checkLogins;
@@ -1156,10 +1271,6 @@ sub get2fTplParams {
     $param{CHECKLOGINS}   = $checkLogins;
     $param{STAYCONNECTED} = $stayConnected;
 
-    $param{AUTH_ERROR}      = $req->error;
-    $param{AUTH_ERROR_TYPE} = $req->error_type;
-    $param{AUTH_ERROR_ROLE} = $req->error_role;
-
     return %param;
 }
 
