@@ -3,6 +3,7 @@ package Lemonldap::NG::Portal::Lib::Choice;
 use strict;
 use Mouse;
 use Safe;
+use Lemonldap::NG::Portal::Main::Constants qw(PE_OK);
 
 extends 'Lemonldap::NG::Portal::Lib::Wrapper';
 with 'Lemonldap::NG::Portal::Lib::OverConf';
@@ -67,16 +68,12 @@ sub init {
         }
 
         # Display conditions
-        my $safe = Safe->new;
         my $cond = $mods[4];
-        if ( defined $cond and $cond !~ /^$/ ) {
-            $self->logger->debug("Found rule $cond for $name");
-            $_choiceRules->{$name} =
-              $safe->reval("sub{my(\$env)=\@_;return ($cond)}");
-            if ($@) {
-                $self->logger->error("Bad condition $cond: $@");
-                return 0;
-            }
+        if ( defined $cond and $cond ne "" ) {    # 0 is a valid rule!
+            my $rule =
+              $self->p->buildRule( $cond, "choice condition for $name" );
+            return 0 unless $rule;
+            $_choiceRules->{$name} = $rule;
         }
         else {
             $self->logger->debug("No rule for $name");
@@ -92,78 +89,13 @@ sub init {
 
 # RUNNING METHODS
 
-sub initDisplay {
-    my ($self, $req, $module) = @_;
-
-    my $mod = $self->modules->{$module};
-    if ( $mod->{AjaxInitScript} ) {
-        $self->logger->debug( 'Append ' . $mod->{Name} . ' init/script' )
-        if $mod->{Name};
-        $req->data->{customScript} .= $mod->AjaxInitScript;
-    }
-    if ( $mod->{InitCmd} ) {
-        $self->logger->debug( 'Launch ' . $mod->{Name} . ' init command' )
-        if $mod->{Name};
-        my $res = eval( $mod->{InitCmd} );
-        if ($@) {
-            die "Error running InitCmd: $@";
-        }
-    }
-
-}
-
 sub checkChoice {
     my ( $self, $req ) = @_;
-    my $name;
 
-    # Check Choice from pdata
-    if ( defined $req->pdata->{_choice} ) {
-        $name = $req->pdata->{_choice};
-        $self->logger->debug("Choice $name selected from pdata");
-    }
-    elsif ( (defined $req->data->{_authChoice}) or (defined $req->data->{_choice}) ) {
-        $name = $req->data->{_authChoice} || $req->data->{_choice};
-        $self->logger->debug("Choice $name selected from req->data");
-    }
+    my ( $name, $how ) = $self->_getChoiceFromReq($req);
 
-    unless ($name) {
-
-        # Check with catch method
-        foreach ( keys %{ $self->catch } ) {
-            if ( $req->path_info =~ $self->catch->{$_} ) {
-                $name = $_;
-                $self->logger->debug(
-                    "Choice $name selected from " . $req->path_info );
-                last;
-            }
-        }
-    }
-
-    unless ($name) {
-
-        # Set by OAuth Resource Owner grant // RESTServer pwdCheck
-        if ( $req->data->{_pwdCheck} and $self->{conf}->{authChoiceAuthBasic} )
-        {
-            $name = $self->{conf}->{authChoiceAuthBasic};
-        }
-    }
-
-    unless ($name) {
-
-        # Check with other methods
-        $name ||=
-             $req->data->{findUserChoice}
-          || $req->param( $self->conf->{authChoiceParam} )
-          || $req->userData->{_choice}
-          || $req->sessionInfo->{_choice}
-          or return 0;
-        my $from =
-            $req->data->{findUserChoice}                  ? 'findUser'
-          : $req->param( $self->conf->{authChoiceParam} ) ? 'param'
-          : $req->userData->{_choice}                     ? 'userData'
-          :                                                 'sessionInfo';
-        $self->logger->debug("Choice $name selected from $from");
-    }
+    return 0 unless ($name);
+    $self->logger->debug("Choice $name selected from $how");
 
     unless ( defined $self->modules->{$name} ) {
         $self->logger->error("Unknown choice '$name'");
@@ -185,6 +117,63 @@ sub checkChoice {
     return $name;
 }
 
+sub _getChoiceFromReq {
+    my ( $self, $req ) = @_;
+
+    # Check Choice from pdata
+    if ( defined $req->pdata->{_choice} ) {
+        return ( $req->pdata->{_choice}, "pdata" );
+    }
+    elsif (( defined $req->data->{_authChoice} )
+        or ( defined $req->data->{_choice} ) )
+    {
+        my $name = $req->data->{_authChoice} || $req->data->{_choice};
+        return ( $name, "req->data" );
+    }
+
+    # Check with catch method
+    foreach ( keys %{ $self->catch } ) {
+        if ( $req->path_info =~ $self->catch->{$_} ) {
+            return ( $_, "caught path " . $req->path_info );
+        }
+    }
+
+    # Set by OAuth Resource Owner grant // RESTServer pwdCheck
+    if ( $req->data->{_pwdCheck} and $self->{conf}->{authChoiceAuthBasic} ) {
+        return ( $self->{conf}->{authChoiceAuthBasic}, "basic auth context" );
+    }
+
+    # Check with other methods
+    my $options = [
+        [ findUser => $req->data->{findUserChoice} ],
+        [ param    => scalar( $req->param( $self->conf->{authChoiceParam} ) ) ],
+        [ userData => $req->userData->{_choice} ],
+        [ sessionInfo => $req->sessionInfo->{_choice} ],
+    ];
+    for my $opt (@$options) {
+        return ( $opt->[1], $opt->[0] )
+          if $opt->[1];
+    }
+
+    if ($self->conf->{authChoiceSelectOnly}) {
+        my @allowed_choices = grep { $self->_evaluateRule( $req, $_ ) }
+        keys %{ $self->conf->{authChoiceModules} };
+        if ( @allowed_choices == 1 ) {
+            return ( $allowed_choices[0], "only available" );
+        }
+    }
+
+    # Try hook
+    my $context = {};
+    my $h       = $self->p->processHook( $req, 'getAuthChoice', $context );
+    return 0 if ( $h != PE_OK );
+    if ( $h == PE_OK and $context->{choice} ) {
+        return ( $context->{choice}, "hook" );
+    }
+
+    return 0;
+}
+
 sub name {
     my ( $self, $req, $type ) = @_;
     unless ($req) {
@@ -225,14 +214,10 @@ sub _buildAuthLoop {
         my ( $auth, $userDB, $passwordDB, $url, $condition ) =
           split( /;\s*/, $self->conf->{authChoiceModules}->{$_} );
 
-        unless ( $_choiceRules->{$_} ) {
-            $self->logger->error("$_ has no rule");
-            $_choiceRules->{$_} = sub { 1 };
-        }
-        unless ( $_choiceRules->{$_}->( $req->env ) ) {
+        unless ( $self->_evaluateRule( $req, $_ ) ) {
             $self->logger->debug(
-"Condition returns false, authentication choice $_ will not be displayed"
-            );
+                    "Condition returns false, authentication choice $_"
+                  . " will not be displayed" );
         }
         else {
             $self->logger->debug("Displaying authentication choice $_");
@@ -242,8 +227,6 @@ sub _buildAuthLoop {
                 $req->data->{cspFormAction} ||= {};
                 if (
                     defined $url
-                    and not $self->checkXSSAttack( 'URI',
-                        $req->env->{'REQUEST_URI'} )
                     and $url =~
                     q%^(https?://)?[^\s/.?#$].[^\s]+$% # URL must be well formatted
                   )
@@ -258,17 +241,24 @@ sub _buildAuthLoop {
                 $self->logger->debug("Use URL $url");
 
                 eval {
-                    $self->_authentication->initDisplay($req, $_);
+                    my $mod = $self->_authentication->modules->{$_};
+                    $mod->initDisplay($req) if $mod->can('initDisplay');
                 };
-                $self->logger->info("Unable to initialize choice $_ display: $@") if $@;
+                $self->logger->info(
+                    "Unable to initialize choice $_ display: $@")
+                  if $@;
 
+                my $name_without_space = $name =~ s/^\s*//r;
 
                 # Options to store in the loop
                 my $optionsLoop = {
-                    name   => $name,
-                    key    => $_,
-                    module => $auth,
-                    url    => $url
+                    name                       => $name,
+                    key                        => $_,
+                    module                     => $auth,
+                    url                        => $url,
+                    "name_$name_without_space" => 1,
+                    "module_$auth"             => 1,
+                    "key_$_"                   => 1,
                 };
 
                 # Get displayType for this module
@@ -316,6 +306,19 @@ sub _buildAuthLoop {
                     }
                 }
 
+                # If a choice has already been selected,
+                # activate the corresponding form
+                if ( $req->data->{_authChoice} ) {
+                    $optionsLoop->{ACTIVE_FORM} =
+                      ( $req->data->{_authChoice} eq $_ );
+
+                }
+
+                # if not, activate the first form in the list
+                else {
+                    $optionsLoop->{ACTIVE_FORM} = @authLoop ? 0 : 1;
+                }
+
                 # Register item in loop
                 push @authLoop, $optionsLoop;
 
@@ -334,5 +337,18 @@ sub _buildAuthLoop {
 
 }
 
+sub _evaluateRule {
+    my ( $self, $req, $choice ) = @_;
+
+    my $extraData = {};
+    $extraData->{targetAuthnLevel} = $req->pdata->{targetAuthnLevel}
+      if defined $req->pdata->{targetAuthnLevel};
+    unless ( $_choiceRules->{$choice} ) {
+        $self->logger->error("$choice has no rule");
+        $_choiceRules->{$choice} = sub { 1 };
+    }
+    return $_choiceRules->{$_}->( $req, $extraData );
+}
+
 1;
 
