@@ -9,10 +9,10 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
 );
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.19.0';
 
 extends qw(
-  Lemonldap::NG::Common::Module
+  Lemonldap::NG::Portal::Main::UserDB
   Lemonldap::NG::Portal::Lib::OpenIDConnect
 );
 
@@ -57,8 +57,13 @@ sub getUser {
     $req->data->{OpenIDConnect_user_info} = $userinfo_content;
 
     # Check that received sub is the same than current user
-    unless ( $req->data->{OpenIDConnect_user_info}->{sub} eq $req->{user} ) {
-        $self->logger->error("Received sub do not match current user");
+    my $id_token_sub =
+      $req->data->{id_token_sub} || $req->userData->{_oidc_sub};
+    my $received_sub = $req->data->{OpenIDConnect_user_info}->{'sub'};
+    unless ( $received_sub eq $id_token_sub ) {
+        $self->logger->error(
+"Received sub $received_sub does not match current user $id_token_sub"
+        );
         return PE_BADCREDENTIALS;
     }
 
@@ -78,19 +83,21 @@ sub setSessionInfo {
 
     my %vars = (
         %{ $self->conf->{exportedVars} },
-        %{ $self->conf->{oidcOPMetaDataExportedVars}->{$op} }
+        %{ $self->opAttributes->{$op} }
     );
 
     while ( my ( $k, $v ) = each %vars ) {
-        $req->{sessionInfo}->{$k} = $req->data->{OpenIDConnect_user_info}->{$v};
+        my $value = $req->data->{OpenIDConnect_user_info}->{$v};
+        if (  ref($value) and ref($value) eq "ARRAY" ) {
+            $req->{sessionInfo}->{$k} =
+              join( $self->conf->{multiValuesSeparator}, @$value );
+        }
+        else {
+            $req->{sessionInfo}->{$k} = $value;
+        }
     }
 
     return PE_OK;
 }
 
-# Does nothing
-sub setGroups {
-    return PE_OK;
-}
-
 1;
